"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * Assign to the first objects the following
 * @param {Object} args the objects list
 * @return {Object} the first object
 */
function assign() {
    var args = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        args[_i] = arguments[_i];
    }
    return Array.prototype.reduce.call(arguments, function (target, source) {
        return Object.keys(Object(source)).reduce(function (t, k) {
            t[k] = source[k];
            return t;
        }, target);
    });
}
exports.assign = assign;
function stringify(value) {
    return value
        .replace(/'/gim, '\\\'')
        .replace(/\n/gi, '\\n');
}
/**
 * Interpolate the given value.
 * @param {string} value the value
 * @param {ParserOptions} options the options
 * @return {string} the interpolated value
 */
function interpolate(value, options) {
    var result;
    var statements = [];
    var lastIndex = 0;
    while ((result = options.interpolation.exec(value)) !== null) {
        var full = result[0];
        var group = result[1];
        var index = result.index;
        var before_1 = value.substring(lastIndex, index);
        if (before_1) {
            statements.push("'" + stringify(before_1) + "'");
        }
        if (group.trim()) {
            statements.push("(" + group.trim() + " === undefined || " + group.trim() + " === null ? '' : " + group.trim() + ")");
        }
        lastIndex = index + full.length;
    }
    var after = value.substring(lastIndex, value.length);
    if (after) {
        statements.push("'" + stringify(after) + "'");
    }
    return statements.join(' + ');
}
exports.interpolate = interpolate;
/**
 * Converts string to camel case.
 * @param {string} value the string to convert
 * @return {string} the camel cased string
 */
function toCamelCase(value) {
    return value.toLowerCase()
        .split('-')
        .map(function (part, index) { return index ? part.charAt(0).toUpperCase() + part.slice(1) : part; }).join('');
}
exports.toCamelCase = toCamelCase;
